module hub (
            input logic         rst,
            input logic         sys_clk,
            input logic [31:0]  rx_cmd,       // for routing-related commands
            input logic [3:0]   rx_cmd_valid,
            input logic [31:0]  rx_byte,
            input logic [3:0]   rx_valid,
            input logic [7:0]   rx2tx_dest,   // rx byte's destination
            input logic [3:0]   tx_ready,     // if tx_byte is ready to be read
            output logic [3:0]  rx_ready,     // if rx_byte is ready to be read
            output logic [7:0]  tx_src,       // tell the tx where the stream is comming from
            output logic [31:0] tx_byte,
            output logic [3:0]  tx_valid,
            output logic [1:0]  packet_size); // 4 states for 4 fixed packet sizes
    timeunit 1ns;
    timeprecision 1ps;
    
    // TBD: pre-agree on packet size

    // use the round-robin strat to poll since the routing is much faster
    // NOTE: To expand to more connected_devices, use a hierarchical design
    logic [1:0]           curr_service = 0;
    logic [1:0]           last_dest = 0;

    // src dest byte
    typedef struct {
        logic [1:0] dest;
        logic [7:0] payload;
    } svc_buffer;
    svc_buffer service_buffer [3:0];
    svc_buffer curr_buffer;
    assign curr_buffer = service_buffer[curr_service];
    logic [3:0]           in_buffer;
    assign rx_ready = ~in_buffer;

    always_ff @ (posedge sys_clk) begin
        if (rst) begin
            in_buffer <= '0;
            tx_src <= '0;
            tx_valid <= '0;
            packet_size <= '0;
            curr_service <= '0;
            last_dest <= '0;
            for (int i = 0; i < 4; i++) begin
                service_buffer[i] <= '0;
            end
        end else begin // if (rst)
            // Handle RX side logic
            for (int i = 0; i < 4; i++) begin
                if (rx_valid[i]) begin
                    if (!in_buffer[i]) begin
                        service_buffer[i].dest <= get_dest(rx2tx_dest, i[1:0]);
                        service_buffer[i].payload <= get_byte(rx_byte, i[1:0]);
                        in_buffer[i] <= 1;
                    end
                end
            end

            // Handle TX side logic
            if (in_buffer[curr_service] && tx_ready[curr_buffer.dest]) begin
                tx_byte[{curr_buffer.dest, 3'b000} +: 8]
                    <= curr_buffer.payload;
                tx_src[{curr_buffer.dest, 1'b0} +: 2]
                    <= curr_service;
                in_buffer[curr_service] <= 0;
                tx_valid[curr_buffer.dest] <= 1;
            end
            tx_valid[last_dest] <= 0;
            last_dest <= service_buffer[curr_service].dest;
            curr_service <= curr_service + 1;
        end // else: !if(rst)
    end // always_ff @ (posedge sys_clk)

endmodule // hub

function automatic logic [1:0] get_dest(input logic [7:0] dest_map,
                                        input logic [1:0] idx);
    return dest_map[{idx, 1'b0} +: 2];
endfunction // get_dest

function automatic logic [7:0] get_byte(input logic [31:0] byte_arr,
                                        input logic [1:0] idx);
    return byte_arr[{idx, 3'b000} +: 8];
endfunction // get_byte
